// ***********
// * Scope.h *
// ***********

// Definitions and Tables 
// 
#include <avr/pgmspace.h>					// for AVM because 16/8 bit Flash

void GLCD_GoTo(uint8_t x,uint8_t y);
void GLCD_WriteString ();					// Clear
void set_relais(void);
void display_voltage(void);					// display of voltage and setting of reais
void display_timing(void);					// display of timing
void display_modus(void);					// display of modus
void show_AC_DC(void);						// display of AC/DC-Input
void set_AC_DC(void);						// set relais of AC/DC-Input
void show_trigger_slope(void);				// display + or - slope

volatile uint16_t time_line;	   			// global: key press detect
volatile uint8_t dataLcdBuffer[101];		// global: data-RAM for display
volatile uint8_t key_press;	   				// global: key press detect
volatile uint8_t x_position;				// global: x-Position (0-99) on display
volatile uint8_t count_V;					// global: Voltage-range up and down per key
volatile uint8_t count_S;					// global: Timing-range  up and down per key
volatile uint8_t mod_sel;					// Function-Modus of Scope
volatile uint8_t single_scan;				// 1 = only single measurement, 0 = measure continues 
volatile uint8_t scan_ready;				// global: Scan 1...100 Pixel ready - wait!
volatile uint8_t single_ready;				// global: manual single scan ready
											// 		   1 = ready -> no Scan  0 = start scanning
uint8_t delay_byte1;						// only IRQ: set from save_delay_byte1 timing!
uint8_t delay_byte2;						// dito.
uint8_t delay_byte3;						// dito.
uint8_t delay_byte4;						// dito.
volatile uint8_t save_delay_byte1;			// global: copy directly, to save time in IRQ-Routine
volatile uint8_t save_delay_byte2;			// dito.
volatile uint8_t save_delay_byte3;			// dito.
volatile uint8_t save_delay_byte4;			// dito.

volatile uint8_t trigger_level;				// global: Level set from main, 255= max./ 0= min.
volatile uint8_t trigger_found;				// = 1 if Trigger found, = 0 if not found
volatile uint8_t trigger_wait;				// 1 = wait for trigger, 0 = no trigger
volatile uint8_t trigger_slope;				// 0 = + trigger, 1 = - trigger
volatile uint8_t read_done;					// read ADC only 1x, done = 1
volatile uint8_t display_mode;				// 0: Start: no display selected
											// 1: graphic display 100 pixel on shot 
											// 2: graphic display single pixel auto
											// 3: graphic display ?
uint8_t AC_DC;								// Input-relais: AC = 0 or DC = 1

//
//------------------------------------------------------------------------------
// Raster Letter size: 10x6 for Scope
//------------------------------------------------------------------------------

static const char  LcdRaster[] PROGMEM =
{ 
255,1,1,1,1,1,1,1,1,3,1,1,1,1,1,1,									// 1. Linie 8 bit broad => 128 Byte per Linie
1,1,1,3,1,1,1,1,1,1,1,1,1,3,1,1,
1,1,1,1,1,1,1,3,1,1,1,1,1,1,1,1,
1,171,1,1,1,1,1,1,1,1,1,3,1,1,1,1,
1,1,1,1,1,3,1,1,1,1,1,1,1,1,1,3,
1,1,1,1,1,1,1,1,1,3,1,1,1,1,1,1,
1,1,1,1,255,


255,0,0,0,0,0,0,0,0,8,0,0,0,0,0,0,									// 2. Linie
0,0,0,8,0,0,0,0,0,0,0,0,0,8,0,0,
0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,0,
8,170,8,0,0,0,0,0,0,0,0,8,0,0,0,0,
0,0,0,0,0,8,0,0,0,0,0,0,0,0,0,8,
0,0,0,0,0,0,0,0,0,8,0,0,0,0,0,0,
0,0,0,8,255,

255,0,0,0,0,0,0,0,0,32,0,0,0,0,0,0,									// 3. Linie
0,0,0,32,0,0,0,0,0,0,0,0,0,32,0,0,
0,0,0,0,0,0,0,32,0,0,0,0,0,0,0,0,
32,170,32,0,0,0,0,0,0,0,0,32,0,0,0,0,
0,0,0,0,0,32,0,0,0,0,0,0,0,0,0,32,
0,0,0,0,0,0,0,0,0,32,0,0,0,0,0,0,
0,0,0,32,255,

255,128,0,128,0,128,0,128,0,192,0,128,0,128,0,128,					// 4. Linie
0,128,0,192,0,128,0,128,0,128,0,128,0,192,0,128,
0,128,0,128,0,128,0,192,0,128,0,128,0,128,0,128,
128,234,128,128,0,128,0,128,0,128,0,192,0,128,0,128,
0,128,0,128,0,192,0,128,0,128,0,128,0,128,0,192,
0,128,0,128,0,128,0,128,0,192,0,128,0,128,0,128,
0,128,0,128,255,

255,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,									// 5. Linie
0,0,0,1,0,0,0,0,0,0,0,0,0,1,0,0,
0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,
0,171,0,0,0,0,0,0,0,0,0,1,0,0,0,0,
0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,1,
0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,
0,0,0,0,255,

255,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,									// 6. Linie
0,0,0,2,0,0,0,0,0,0,0,0,0,2,0,0,
0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,
2,170,2,0,0,0,0,0,0,0,0,2,0,0,0,0,
0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,2,
0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,
0,0,0,2,255,

255,0,0,0,0,0,0,0,0,8,0,0,0,0,0,0,									// 7. Linie
0,0,0,8,0,0,0,0,0,0,0,0,0,8,0,0,
0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,0,
8,170,8,0,0,0,0,0,0,0,0,8,0,0,0,0,
0,0,0,0,0,8,0,0,0,0,0,0,0,0,0,8,
0,0,0,0,0,0,0,0,0,8,0,0,0,0,0,0,
0,0,0,8,255,

255,128,128,128,128,128,128,128,128,160,128,128,128,128,128,128, 	// 8. Linie
128,128,128,160,128,128,128,128,128,128,128,128,128,160,128,128,
128,128,128,128,128,128,128,160,128,128,128,128,128,128,128,128,
160,170,160,128,128,128,128,128,128,128,128,160,128,128,128,128,
128,128,128,128,128,160,128,128,128,128,128,128,128,128,128,160,
128,128,128,128,128,128,128,128,128,160,128,128,128,128,128,128,
128,128,128,128,255};

//------------------------------------------------------------------------------

static const char timing [25][10] PROGMEM =	// [linie][column] of Table in Flash-ROM
											// for 16 MHz Quarz
{
//|--value--|    |--unity--|	byte1 byte2 byte3 byte4	  step   |--Range--|

{'1', '0', '0', ' ', 'u', 's', 	0x00, 0x00, 0x00, 0x00}, //	 0   100  us/Skt -------
{'2', '0', '0', ' ', 'u', 's', 	0x01, 0x00, 0x00, 0x00}, //	 1   200  us/Skt	|
{'5', '0', '0', ' ', 'u', 's', 	0x04, 0x00, 0x00, 0x00}, //	 2   500  us/Skt display  
{' ', ' ', '1', ' ', 'm', 's', 	0x09, 0x00, 0x00, 0x00}, //	 3	   1  ms/Skt if scan
{' ', ' ', '2', ' ', 'm', 's', 	0x13, 0x00, 0x00, 0x00}, //	 4     2  ms/Skt complete
{' ', ' ', '5', ' ', 'm', 's', 	0x31, 0x00, 0x00, 0x00}, //	 5     5  ms/Skt 	|
{' ', '1', '0', ' ', 'm', 's', 	0x63, 0x00, 0x00, 0x00}, //	 6    10  ms/Skt	|
{' ', '2', '0', ' ', 'm', 's', 	0x63, 0x01, 0x00, 0x00}, //	 7    20  ms/Skt -------

{' ', '5', '0', ' ', 'm', 's', 	0x63, 0x04, 0x00, 0x00}, //	 8    50  ms/Skt -------
{'1', '0', '0', ' ', 'm', 's', 	0x63, 0x09, 0x00, 0x00}, //	 9   100  ms/Skt	|
{'2', '0', '0', ' ', 'm', 's', 	0x63, 0x13, 0x00, 0x00}, //	10   200  ms/Skt	|
{'5', '0', '0', ' ', 'm', 's', 	0x63, 0x31, 0x00, 0x00}, //	11   500  ms/Skt	|
{' ', ' ', '1', 's', 'e', 'c', 	0x63, 0x63, 0x00, 0x00}, //	12	   1 sec/Skt	|
{' ', ' ', '2', 's', 'e', 'c', 	0x63, 0x63, 0x01, 0x00}, //	13     2 sec/Skt display
{' ', ' ', '5', 's', 'e', 'c', 	0x63, 0x63, 0x04, 0x00}, //	14     5 sec/Skt each
{' ', '1', '0', 's', 'e', 'c', 	0x63, 0x63, 0x09, 0x00}, //	15    10 sec/Skt point
{' ', '2', '0', 's', 'e', 'c', 	0x63, 0x63, 0x13, 0x00}, //	16    20 sec/Skt immediately
{' ', '5', '0', 's', 'e', 'c', 	0x63, 0x63, 0x31, 0x00}, //	17    50 sec/Skt	|
{'1', '0', '0', 's', 'e', 'c', 	0x63, 0x63, 0x63, 0x00}, //	18   100 sec/Skt	|
{'2', '0', '0', 's', 'e', 'c', 	0x63, 0x63, 0x63, 0x01}, //	19   200 sec/Skt	|
{' ', '1', '0', 'm', 'i', 'n', 	0x63, 0x63, 0x63, 0x05}, //	20	  10 min/Skt	|
{' ', '2', '0', 'm', 'i', 'n', 	0x63, 0x63, 0x63, 0x0B}, //	21    20 min/Skt	|
{' ', '5', '0', 'm', 'i', 'n', 	0x63, 0x63, 0x63, 0x1D}, //	22    50 min/Skt	|
{'1', '0', '0', 'm', 'i', 'n', 	0x63, 0x63, 0x63, 0x3B}, //	23   100 min/Skt	|
{'2', '0', '0', 'm', 'i', 'n', 	0x63, 0x63, 0x63, 0x77}	 //	24   200 min/Skt --------

};

//------------------------------------------------------------------------------

static const char gain_relais[7] PROGMEM =		// Relais-Table in Flash
{
// 		relais									decimal
        0x70,		//    5mV/Skt	gain 100	= 112
        0x60,		//   10mV/Skt	gain  50	=  96
        0x50,		//   25mV/Skt	gain  20	=  80
        0x40,		//	 50mV/Skt	gain  10	=  64
        0x20,		//  100mV/Skt	gain   5	=  32
        0x10,		//  250mV/Skt	gain   2	=  16
        0x00		//  500mV/Skt	gain   1	=   0

};

static const uint16_t gain_display[7] PROGMEM =	// Display-Table in Flash
{
// 		value
      	 5,			// (0)   5mV/Skt	gain 100
       	10,			// (1)  10mV/Skt	gain  50
        25,			// (2)  25mV/Skt	gain  20
        50,			// (3)  50mV/Skt	gain  10
       100,			// (4) 100mV/Skt	gain   5
       250,			// (5) 250mV/Skt	gain   2
       500			// (6) 500mV/Skt	gain   1


};

//------------------------------------------------------------------------------

static const char modus [4][4] PROGMEM =		// [linie][column] of Modus-Table in Flash-ROM
{
// 		  |----Modus-----|

        {'A', 'u', 't', 'o'},	//	0	Auto Trigger - toggle AC/DC input
		{'C', 'o', 'n', 't'},	//	1	display continues without Trigger
 		{'S', 'i', 'n', 'g'},	//	2	Single Trigger - show Start/Stop
        {'T', 'r', 'i', 'g'},	//	3	Set Trigger Level with UP/Down Key!

//     	{'M', 'e', 'a', 's'},	//	4	Measure !
//     	{'P', 'r', 'i', 'n'}	//	5	Print !
};



//===============================================================================
//  FEATURES of Oscilloscope
//===============================================================================

// byte = pgm_read_byte(&(mydata[i][j]));					//  read Prog. Memory

// use ATMEGA 16 16MHz ... ATMEGA 644 20MHz
/* Timebase with Quarz 16MHz - Timer 1:  fCLK_I/O + Software =>10us IRQ

	/Pkt	/Skt	/SE		byte1	byte2	byte3	byte4
------------------------------------------------------------------

	 10us	100us	  1ms	  0		  0		  0		  0
	 20us	200us	  2ms	  1		  0		  0		  0
	 50us	500us	  5ms	  4		  0		  0		  0
	100us	  1ms	 10ms	  9		  0		  0		  0
	200us	  2ms	 20ms	 19		  0		  0		  0
	500us	  5ms	 50ms	 49		  0		  0		  0

	  1ms	 10ms	100ms	 99		  0		  0		  0
	  2ms	 20ms	200ms	 99		  1		  0		  0
	  5ms	 50ms	500ms	 99		  4		  0		  0
	 10ms	100ms	  1 s	 99		  9		  0		  0
	 20ms	200ms	  2 s	 99		 19		  0		  0
	 50ms	500ms	  5 s	 99		 49		  0		  0

	100ms	  1 s	 10 s	 99		 99		  0		  0
	200ms	  2 s	 20 s	 99	 	 99		  1		  0
	500ms	  5 s	 50 s	 99		 99		  4		  0
	   1s	 10 s	100 s	 99		 99		  9		  0
	   2s	 20 s	200 s	 99		 99		 19		  0
	   5s	 50 s	500 s	 99		 99		 49		  0

	  10s	100 s  1000 s	 99		 99		 99		  0
	  20s	200 s  2000 s	 99	 	 99		 99		  1
	   1m	 10 m   100 m	 99		 99		 99		  5
	   2m	 20 m	200 m	 99		 99		 99		 11
	   5m	 50 m	500 m	 99		 99		 99		 19
	  10m	100 m  1000 m	 99		 99		 99		 59
	  20m	200 m  2000 m	 99		 99		 99		119		(=33,3 Std.)

*/


//===============================================================================


/*  Eingangsverstrker *1 ...*100

	PA7 = relais 1	AC/DC
	PA6 = relais 2	*10
	PA5 = relais 3	*5
	PA4 = relais 4	*2


	pro/Skt	  SE	PA7 (=DC/AC)	PA6	PA5	PA4	hex	
			[Vss]					*10	*5	*2
	---------------------------------------------------

      5mV 	 30mV	1/0				1	1	1	0x70	
	 10mV	 60mV	1/0 			1	1	0	0x60
     25mV	150mV	1/0 			1	0	1	0x50
	 50mV	300mV	1/0 			1	0	0	0x40
	100mV	600mV	1/0				0	1	0	0x20
	250mV	 1,5V	1/0				0	0	1	0x10
	500mV	 3  V	1/0				0	0	0	0x00

					1 = relais on
					0 = relais off
*/


//===============================================================================

/*
Timing AD-Wandler:

	datasheet: 13us(1MHz ADC Clock at 13 Cycles) up to 260us

	=> 	Quarz 16MHz :32 bei ADC-Teiler/8 und 14 Takte ~ 7 us => <10us for digitizing and  storage in RAM

	=>  10us Interrupt and 160 Takte Zeit for IRQ-Routine at 8 Bit ADC
*/

