//-------------------------------------------------------------------------------------------------
//
// OSCILLOSCOPE with broad timing range
// Display: 128x64 pixel
// Timing range: 10us/Div ... 30 min/Div
// Input range:   5mV/Div ... 500mV/Div
// Multible timing modes
//
// Scope functions added by Karl-Anton Dichtel
// August 2011
//
// libary for LCD-Display of
// (c) Radoslaw Kwiecie, radek@dxp.pl
// http://en.radzio.dxp.pl
// Translation: http://translate.google.de/#pl|de|
// 				http://de.bab.la/woerterbuch/polnisch-deutsch
//
// Debouncing 8 Keys
// Peter Fleury <pfleury@gmx.ch> http://jump.to/fleury,
// based on algorithm of Peter Dannegger <danni@specs.de>
//
// Hardware definition: KS0108.h  	 Size: 128/192 Pixel
//						KS0108-AVR.c Port definitions LCD-display
//						Scope.c		 Port definitions relais and Key
//
//-------------------------------------------------------------------------------------------------

#include "KS0108.h"						// Definitions for KS108
#include "graphic.h"					// Call of Funktions
#include "main.h"						// Call of Funktions
#include "font5x8.h"					// Table with Font
#include "Scope.h"						// Definitions and Tables 
#include "stdlib.h"						// additional Lib
#include "string.h"						// additional Lib
#define F_CPU 16000000UL				// 
#include "util/delay.h"					// additional Lib


//-------------------------------------------------------------------------------------------------
// MAIN PRORAMM
//-------------------------------------------------------------------------------------------------

int main(void)							// Start MAIN
{
										// Initialize:
	GLCD_Initalize();					// Display-Controller
	GLCD_ClearScreen();					// Screen
	ADC_init ();						// ADC on PA0
//	KeyInit();							// later!

//-------------------------------------------------------------------------------------------------

	save_delay_byte1 = pgm_read_byte(&timing[count_S][6]);	// | Read and store
	save_delay_byte2 = pgm_read_byte(&timing[count_S][7]);	// | Timing-Range Table:
	save_delay_byte3 = pgm_read_byte(&timing[count_S][8]);	// | 200us...10msec /SKT
	save_delay_byte4 = pgm_read_byte(&timing[count_S][9]);	// | and store it

//-------------------------------------------------------------------------------------------------

	// mark right border of display (for 128x64 Pixel range)
//	GLCD_Line(128,0,128, 63);	
	
//-------------------------------------------------------------------------------------------------		

	GLCD_GoTo(104,0);					// No. of pixel: 0, 1, .. , No. of Linie 0, 1,..
	GLCD_WriteString("Oszi");  			// (One linie = 32 Letter x 6 Pixel)

	GLCD_Bitmap((char *) (int)LcdRaster,0,0,101,64);	// draw background for Scope-Display

	GLCD_GoTo(110,2);
	GLCD_WriteString ("mV");			// Voltage range, here allways mV

	GLCD_GoTo(104,5);
	GLCD_WriteString ("/Skt");			// Display of "/Skt" for voltage and timebase

	GLCD_GoTo(116,7);					// for Modus AUTO => Input-relais off = AC
	GLCD_WriteString ("AC");

/*  For information only:
	char Modus [] = "Auto";				// Input of  Strings with = definition!!!
	GLCD_GoTo(104,7);
	GLCD_WriteString (Modus);
	GLCD_SetPixel(5,45,1);				// (unsigned char x, unsigned char y, unsigned char color)
	GLCD_TogglePixel(5,46,1);			// (unsigned char x, unsigned char y, unsigned char color)
	GLCD_GoTo(114,7);					// (Column-Pixel,Line)
	itoa (trigger_level/4, AsciiZeichen, 10);
	GLCD_WriteString (AsciiZeichen);	// (char charToWrite)	
	_delay_ms(1000);
*/

//-------------------------------------------------------------------------------------------------
// Start main-loop
//-------------------------------------------------------------------------------------------------

	KeyInit();					// Keyboard-Interupt and enable IRQ (=> SEI)
   
while (1)						// do it always
	{

	KeyScan();					// perform pressed keys from IRQ

//-------------------------------------------------------------------------------------------------
// Display: one cycle or each pixel immediately
//-------------------------------------------------------------------------------------------------

	if(!(mod_sel == 2) || single_scan == 1)		// stop displaying after single scan
		{
	
		if(count_S < 8)			// Display all pixel in one Cycle: 100us ... 20ms
			{
			display_one_line();
			}

		else					// Display each pixel directly: 20ms ... 200min
			{
			display_one_pixel();
			}

//			single_scan = 0;	// one "single scan" is finished
				
		}

	}		// End of while
}			// End of MAIN


//-------------------------------------------------------------------------------------------------
// Display Pixel: x = 0-100 / y = 63 - 0 
//-------------------------------------------------------------------------------------------------

	// Display all pixel in one Cycle

		uint8_t scan_displ;								// tougle Scan: 1 = write  0 = clear
		uint8_t i;										// Index for loops

		void display_one_line(void)
			{

			if(!(display_mode == 1))					// only 1x if display-mode has changed
				{
				GLCD_Bitmap((char *) (int)LcdRaster,0,0,101,64);// clear Raster for Scope-Display
				display_mode = 1;	
				scan_displ = 1;							// tougle Scan: 1 = write
				x_position = 0;	
				}


			if (scan_ready == 1 && time_line == 0)		// wait until ADC-scan is ready 
				{										// and display-time is over

	   			for (i=1; i < 101; i++)					// output complete display (100 Pixel)
					{
					GLCD_TogglePixel(i,(255-(dataLcdBuffer[i]))/4,1);	// Toggle 1 Pixel: on/ or off/on
					}													// inverse for display!
				
				// show signal on display for 0.5 sec
					if(scan_displ == 1 && !(mod_sel == 2))
						{
						time_line = 50000;				// set display delay = 0.5 sec
						scan_displ = 0;					// do not change display-delay
						}

				// prepare actual timing and allow measuremant by ADC
					else								// preparare aktuell timing constants
						{								// 
						if(mod_sel == 2)				// single scan mode only:
							{
							single_scan = 0;			// one "single scan" done
							GLCD_GoTo(104,7);			// (Column-Pixel,Line)
							GLCD_WriteString ("Stop");	// display "Stop"
							}
						else
							scan_ready = 0;				// allow next Scan of ADC
							scan_displ = 1;				// allow display of measured values	
						}
				}
			}

//-------------------------------------------------------------------------------------------------

	// Display each pixel directly

		uint8_t j;										// Index for last displayed pixel
		uint8_t first_scan;								// show first scan after display is deleted	

		void display_one_pixel(void)
			{			
						
			if(!(display_mode == 2))					// only 1x if display-mode has changed
				{

				GLCD_Bitmap((char *) (int)LcdRaster,0,0,101,64);// clear Raster for Scope-Display
				display_mode = 2;						// graphic display single pixel auto
				j = 1;									// set no of displayed pixel (1..100)
				first_scan = 1;							// show first scan after display is deleted
				scan_ready=0;							// Start again at Pixel 1
				x_position = 0;							// Start of scan
				}

				if ((j <= x_position) && (read_done ==1)) // only 1x if value measured

					{
					if(!(first_scan == 1))
					GLCD_TogglePixel(x_position,(255-(dataLcdBuffer[x_position]))/4,1);	// clear first pixel

					GLCD_TogglePixel(x_position-1,(255-(dataLcdBuffer[x_position-1]))/4,1);	//  set one Pixel
					j++;
					}

					if(j == 101)						// last pixel in linie set
						{
	
						first_scan = 0;					// delete pixel, bevor write 
						j = 1;							// Start display from beginning
						x_position = 0;					// Start ADC from beginning

						if(mod_sel == 2)				// single scan mode
							{
							single_scan = 0;			// one "single scan" done
							GLCD_GoTo(104,7);			// (Column-Pixel,Line)
							GLCD_WriteString ("Stop");	// display "Stop"
							}
						else
							scan_ready = 0;				// allow next Scan of ADC
						}					
			}
